@@ -140,38 +140,45 @@ function raiseTab(tabId) {
 // ------------------------------------------------------------------------
 //
 
+var kdeConnectMenuIdPrefix = "kdeconnect_page_";
+
+chrome.contextMenus.onClicked.addListener(function (info) {
+    if (!info.menuItemId.startsWith(kdeConnectMenuIdPrefix)) {
+        return;
+    }
+
+    var deviceId = info.menuItemId.substr(kdeConnectMenuIdPrefix.length);
+
+    var url = info.linkUrl || info.pageUrl;
+    console.log("Send url", url, "to kdeconnect device", deviceId);
+    if (!url) {
+        return;
+    }
+
+    port.postMessage({
+        subsystem: "kdeconnect",
+        event: "shareUrl",
+        url: url,
+        deviceId: deviceId
+    });
+});
+
 addCallback("kdeconnect", "deviceAdded", function(message) {
     var id = message.id;
     var name = message.name;
 
     var menuEntryTitle = chrome.i18n.getMessage("kdeconnect_open_device", name);
-    var menuId = "kdeconnect_page_" + id;
+    var menuId = kdeConnectMenuIdPrefix + id;
 
     chrome.contextMenus.create({
         id: menuId,
         contexts: ["link", "page"],
         title: menuEntryTitle,
     });
-
-    chrome.contextMenus.onClicked.addListener(function (info) {
-        if (info.menuItemId == menuId) {
-            var url = info.linkUrl || info.pageUrl;
-            console.log("Send url", url, "to kdeconnect device", id);
-            if (!url) {
-                return;
-            }
-            port.postMessage({
-                subsystem: "kdeconnect",
-                event: "shareUrl",
-                url: url,
-                deviceId: id
-            });
-        }
-    });
 });
 
 addCallback("kdeconnect", "deviceRemoved", function(message) {
-    chrome.contextMenus.remove("kdeconnect_page_" + id)
+    chrome.contextMenus.remove("kdeconnect_page_" + message.id)
 });
 
 // MPRIS
@@ -405,7 +412,7 @@ var whitelistedDownloadProperties = [
     "id", "url", "finalUrl", "filename", "startTime", "estimatedEndTime", "totalBytes", "bytesReceived", "state", "error", /*"canResume"*/, "paused"
 ];
 
-chrome.downloads.onCreated.addListener(function (download) {
+function createDownload(download) {
     // don't bother telling us about completed downloads...
     // otherwise on browser startup we'll spawn a gazillion download progress notification
     if (download.state === "complete" || download.state === "interrupted") {
@@ -418,13 +425,39 @@ chrome.downloads.onCreated.addListener(function (download) {
     startSendingDownloadUpdates();
 
     port.postMessage({subsystem: "downloads", event: "created", download: filteredDownload});
-});
+}
+
+function sendDownloads() {
+    // When extension is (re)loaded, create each download initially
+    chrome.downloads.search({
+        state: 'in_progress',
+    }, function (results) {
+        results.forEach(createDownload);
+    });
+}
+
+chrome.downloads.onCreated.addListener(createDownload);
 
 chrome.downloads.onChanged.addListener(function (delta) {
     if (activeDownloads.indexOf(delta.id) === -1) {
         return;
     }
 
+    // An interrupted download was resumed. When a download is interrupted, we finish (and delete)
+    // the job but the browser re-uses the existing download, so when this happen,
+    // pretend a new download was created.
+    if (delta.state) {
+        if (delta.state.previous === "interrupted" && delta.state.current === "in_progress") {
+            console.log("Resuming previously interrupted download, pretending a new download was created");
+            chrome.downloads.search({
+                id: delta.id
+            }, function (downloads) {
+                createDownload(downloads[0]);
+            });
+            return;
+        }
+    }
+
     var payload = {};
 
     whitelistedDownloadProperties.forEach(function (item) {
@@ -666,6 +699,7 @@ function connectHost() {
 
     sendEnvironment();
     sendSettings();
+    sendDownloads();
 }
 
 addRuntimeCallback("settings", "changed", function () {
